\name{gpcm}
\alias{gpcm}
\alias{update.gpcm.em}
\alias{update.gpcm.nr}
\alias{print.gpcm}

%- Also NEED an '\alias' for EACH other topic documented here.
\title{Perform MML estimation of the Generalized Partial Credit Model}
\description{\code{gpcm} estimates the genearlized partial credit model by
marginal maximum likelihood.  The estimation is carried out using
Gauss-Hermite quadrature, as implemented in the 'statmod' package, to
approximate the necessary integrals.  The functions
\code{update.gpcm.em} and \code{update.gpcm.nr} are the workhorses for
\code{gpcm}.  They update the parameter estimates by EM and
Newton-Raphson respectively.
}
\usage{
gpcm(X, prior.dist = "norm", tol1 = 1e-04, tol2 = 0.5 * tol1, n.quad.pts = 41, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{X}{\code{X} is a N x J matrix of item responses, where the
  rows represent inidividuals and the column represent items.}
  \item{prior.dist}{The mixing distribution used for the latent
  propensities.  The options are those available in the
  guass.quad.prob function in the statmod package.}
  \item{tol1}{How precise should estimates be before moving from 9
  quadrature points to \code{n.quad.pts}.}
  \item{tol2}{The final tolerance level used to determine
  convergence. }
  \item{n.quad.pts}{The number of quadrature points to use in order to
  approximate various integrals.}
  \item{\dots}{Arguments to pass to \code{gauss.quad.prob}.}
}
\details{
}
\value{
  \code{gpcm} returns a list of type 'gpcm' that contains the following
  elements. 
  \item{estimates}{A matrix containing the parameter estimates.}
  \item{se}{The standard errors of the estimates.}
  \item{data.unique}{A matrix containing the unique response patterns
  in the data set.}
  \item{cts}{The number of individuals with each response pattern.}
  \item{prior.dist}{The mixing distribution used.}
  \item{call}{The function call.}
  \item{log.like}{The value of the log-likelihood at the maximum.}
  \item{eap}{A vector of expected a-posteriori estimates of the latent
  propensity for each row of the data.unique matrix.}
  ...
}
%\references{ ~put references to the literature/web site here ~ }
\author{Matthew S. Johnson}
%\note{ ~~further notes~~ 
%
% ~Make other sections like Warning with \section{Warning }{....} ~
%}
\seealso{ltm,gauss.quad.prob, \code{\link{help}}}
\examples{
a = rep(1,10)
b = seq(-2,2,len=10)
d = matrix(0,10,2)
# Create a randomly generated data set #
X = rgpcm(1000,a,b,d)
X.fit = gpcm(X)

# Print the parameter estimates #
X.fit

# Plot the estimated category response functions #
plot(X.fit)

# Plot the estimated item response functions #
plot(X.fit,plot.type='irf')


## The function is currently defined as
function(X,prior.dist='norm',tol1=1e-4,tol2=0.5*tol1,
  n.quad.pts=41,...){
  # X is an N x J matrix of item responses
  # prior.dist is the prior density function for theta
  # ..., are parameters passed to the prior density
  # Choices are determined by the guass.quad.prob function
  # in library(statmod)---uniform, norm, beta, gamma
  # for the time being the parameters of the prior dist are
  # fixed constants (not estimated).
  
  # Constants 
  K = apply(X,2,max,na.rm=T)
  J = ncol(X)
  N = nrow(X)

  ###################################################################
  # Initialize parameter values #
  ###################################################################
  a = rep(.1,J)
  b = rep(0,J)
  d = matrix(0,J,max(K)) 

  for(i in 1:J){
    if(K[i]<max(K))
      is.na(d[i,(K[i]+1):max(K)]) = T
  }
  # the matrix d is J x max(K), where NAs are used if a given item
  # has fewer than max(K)+1 score categories, maybe inefficient use
  # of memory, but simplifies calculations later

  ###################################################################


  ###################################################################
  # Find unique response patterns and count
  # This part cannot handle NAs
  ####################################################################
  U = as.matrix(unique(as.data.frame(X)))
  n = table(as.data.frame(X))
  n = n[1+U\%*\%c(1,cumprod(K[-J]+1))]
  ####################################################################
  
  GausHerm = gauss.quad.prob(n=9,dist=prior.dist)
  th = GausHerm$node
  wt = GausHerm$w

  ab = cbind(a,b,d)
  flag = FALSE
  while(T){
    while(T){
      # This part uses the gradient-EM approach#
      old = ab
      ab = update.gpcm.em(th,U,a,b,d,wt,n)
      a = ab[,1]
      b = ab[,2]
      d = ab[,-(1:2)]
      #print(rbind(a,b))
      if(max(abs(ab-old),na.rm=T)<tol1) break;
    }
    if(flag ==TRUE) break;
    GausHerm = gauss.quad.prob(n=n.quad.pts,dist=prior.dist,...)
    th = GausHerm$node
    wt = GausHerm$w
    tol1 = tol2
    flag = TRUE
  }
  while(T){
    # This part uses Newton-Raphson (without EM) to update #
    # It also gets an estimate of the standard errors of the ests #
    old = ab
    out = update.gpcm.nr(th,U,a,b,d,wt,n)
    ab = out$est
    a = ab[,1]
    b = ab[,2]
    d = ab[,-(1:2)]
    #print(rbind(a,b))
    if(max(abs(ab-old),na.rm=T)<tol2) break;
  }
  d = as.data.frame(d)
  names(d)=paste('Cat',1:max(K))
  parms = data.frame(Slope=a,Intercept=b,d)
  row.names(parms)=paste('Item',1:J)
  log.like = sum(n*log(out$p))
  eap = out$p.x\%*\%th/out$p
  out = list(estimates = parms, se = out$se, data.unique = U,
    cts = n, prior.dist = prior.dist,
    call = match.call(),log.like=log.like,eap=eap)
  out$se = as.data.frame(out$se)
  row.names(out$se) = row.names(parms)
  names(out$se) = paste('SE.',c('a','b',paste('d',1:max(K),sep='')),sep='')
  class(out) = c('gpcm','data.frame')
  out
  }
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
